@@ -49,14 +49,17 @@ typedef struct
   struct wi_device *device;
   guint timer_id;
 
-  guint state;
+  gint state; /* can be -1 for disconnected devices */
 
   gboolean autohide;
   gboolean autohide_missing;
   gboolean signal_colors;
   gboolean show_icon;
+  gboolean show_bar;
+  gchar *command;
 
   int size;
+  int signal_strength;
   GtkOrientation orientation;
 
   GtkWidget *box;
@@ -69,14 +72,88 @@ typedef struct
 #endif
 
   XfcePanelPlugin *plugin;
-  
+
 } t_wavelan;
 
+enum icon_values {
+    OFFLINE = 0,
+    EXCELLENT,
+    GOOD,
+    OK,
+    WEAK,
+    NONE,
+    INIT,
+    ICON_NUM
+};
+
+char* strength_to_icon[ICON_NUM];
+
 static void wavelan_set_size(XfcePanelPlugin* plugin, int size, t_wavelan *wavelan);
 static void wavelan_set_orientation(XfcePanelPlugin* plugin, GtkOrientation orientation, t_wavelan *wavelan);
+static void wavelan_refresh_icons(t_wavelan *wavelan);
+static void wavelan_update_icon(t_wavelan *wavelan);
+static void wavelan_update_signal(t_wavelan *wavelan);
 
 static void
-wavelan_set_state(t_wavelan *wavelan, gint state)
+wavelan_refresh_icons(t_wavelan *wavelan)
+{
+  GtkIconTheme* theme = gtk_icon_theme_get_default();
+
+  if (gtk_icon_theme_has_icon(theme, "network-wireless-signal-excellent-symbolic"))
+  {
+    strength_to_icon[EXCELLENT] = "network-wireless-signal-excellent-symbolic";
+    strength_to_icon[GOOD] = "network-wireless-signal-good-symbolic";
+    strength_to_icon[OK] = "network-wireless-signal-ok-symbolic";
+    strength_to_icon[WEAK] = "network-wireless-signal-weak-symbolic";
+    strength_to_icon[NONE] = "network-wireless-signal-none-symbolic";
+    strength_to_icon[OFFLINE] = "network-wireless-offline-symbolic";
+  }
+  else /* fallback in case symbolic themes aren't present */
+  {
+    strength_to_icon[EXCELLENT] = "network-wireless-signal-excellent";
+    strength_to_icon[GOOD] = "network-wireless-signal-good";
+    strength_to_icon[OK] = "network-wireless-signal-weak";
+    strength_to_icon[WEAK] = "network-wireless-signal-low";
+    strength_to_icon[NONE] = "network-wireless-signal-none";
+    strength_to_icon[OFFLINE] = "network-wireless-offline";
+  }
+  strength_to_icon[INIT] = strength_to_icon[OFFLINE];
+
+  if (wavelan->signal_strength != INIT) /* only wavelan_new sets INIT */
+    gtk_image_set_from_icon_name(GTK_IMAGE(wavelan->image), strength_to_icon[wavelan->signal_strength], GTK_ICON_SIZE_BUTTON);
+}
+
+static void
+wavelan_update_icon(t_wavelan *wavelan)
+{
+  int signal_strength_prev = wavelan->signal_strength;
+
+  if (!wavelan->show_icon) {
+    gtk_widget_hide(wavelan->image);
+    return;
+  }
+
+  if (wavelan->state > 80)
+    wavelan->signal_strength = EXCELLENT;
+  else if (wavelan->state > 55)
+    wavelan->signal_strength = GOOD;
+  else if (wavelan->state > 30)
+    wavelan->signal_strength = OK;
+  else if (wavelan->state > 5)
+    wavelan->signal_strength = WEAK;
+  else if (wavelan->state >= 0)
+    wavelan->signal_strength = NONE;
+  else
+    wavelan->signal_strength = OFFLINE; /* also for disconnected interfaces */
+
+  if (signal_strength_prev != wavelan->signal_strength)
+    gtk_image_set_from_icon_name(GTK_IMAGE(wavelan->image), strength_to_icon[wavelan->signal_strength], GTK_ICON_SIZE_BUTTON);
+
+  gtk_widget_show(wavelan->image);
+}
+
+static void
+wavelan_update_signal(t_wavelan *wavelan)
 {  
   GdkRGBA color;
   gchar signal_color_bad[] = "#e00000";
@@ -91,27 +168,24 @@ wavelan_set_state(t_wavelan *wavelan, gint state)
     cssminsizes = "min-width: 0px; min-height: 4px";
 #endif
 #endif
-  
-  /* state = 0 -> no link, =-1 -> error */
-  DBG ("Entered wavelan_set_state, state = %d", state);
 
-  if(state > 100)
-    state = 100;
+  if (!wavelan->show_bar) {
+    gtk_widget_hide(wavelan->signal);
+    return;
+  }
 
-  wavelan->state = state;
-
-  if (state >= 1)
-   gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(wavelan->signal), (gdouble) state / 100);
+  if (wavelan->state >= 1)
+   gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(wavelan->signal), (gdouble) wavelan->state / 100);
   else
    gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(wavelan->signal), 0.0);
 
   if (wavelan->signal_colors) {
      /* set color */
-   if (state > 70)
+   if (wavelan->state > 80)
     gdk_rgba_parse(&color, signal_color_strong);
-   else if (state > 55)
+   else if (wavelan->state > 55)
     gdk_rgba_parse(&color, signal_color_good);
-   else if (state > 40)
+   else if (wavelan->state > 30)
     gdk_rgba_parse(&color, signal_color_weak);
    else
     gdk_rgba_parse(&color, signal_color_bad);
@@ -152,11 +226,25 @@ wavelan_set_state(t_wavelan *wavelan, gint state)
   g_free(css);
 #endif
 
-  /* hide icon */
-  if (wavelan->show_icon)
-    gtk_widget_show(wavelan->image);
-  else
-    gtk_widget_hide(wavelan->image);
+  gtk_widget_show(wavelan->signal);
+}
+
+static void
+wavelan_set_state(t_wavelan *wavelan, gint state)
+{
+  /* state = 0 -> no link, =-1 -> error */
+  DBG ("Entered wavelan_set_state, state = %d", state);
+
+  if(state > 100)
+    state = 100;
+
+  wavelan->state = state;
+
+  /* update signal to reflect state */
+  wavelan_update_signal(wavelan);
+
+  /* update icon to reflect state */
+  wavelan_update_icon(wavelan);
 
   /* hide if no network & autohide or if no card found */
   if (wavelan->autohide && state == 0)
@@ -297,13 +385,22 @@ wavelan_read_config(XfcePanelPlugin *plugin, t_wavelan *wavelan)
     {
       if ((s = xfce_rc_read_entry (rc, "Interface", NULL)) != NULL) 
       {
+        if (wavelan->interface)
+          g_free (wavelan->interface);
         wavelan->interface = g_strdup (s);
       } 
-      
       wavelan->autohide = xfce_rc_read_bool_entry (rc, "Autohide", FALSE);
       wavelan->autohide_missing = xfce_rc_read_bool_entry(rc, "AutohideMissing", FALSE);
       wavelan->signal_colors = xfce_rc_read_bool_entry(rc, "SignalColors", FALSE);
       wavelan->show_icon = xfce_rc_read_bool_entry(rc, "ShowIcon", FALSE);
+      wavelan->show_bar = xfce_rc_read_bool_entry(rc, "ShowBar", FALSE);
+      if ((s = xfce_rc_read_entry (rc, "Command", NULL)) != NULL)
+      {
+        if (wavelan->command)
+          g_free (wavelan->command);
+        wavelan->command = g_strdup (s);
+      }
+      xfce_rc_close (rc);
     }
   }
 
@@ -322,20 +419,50 @@ static gboolean tooltip_cb( GtkWidget *widget, gint x, gint y, gboolean keyboard
 	return TRUE;
 }
 
+static void
+wavelan_icon_clicked(GtkWidget *widget, gpointer data,t_wavelan *wavelan)
+{
+  GError    *error = NULL;
+  GtkWidget *message_dialog;
+
+  if (wavelan->command == NULL || strlen(wavelan->command) == 0)
+    return;
+
+  if (!xfce_spawn_command_line_on_screen (gtk_widget_get_screen (GTK_WIDGET (widget)),
+                                            wavelan->command,
+                                          FALSE, FALSE, &error))
+    {
+      message_dialog = gtk_message_dialog_new_with_markup (NULL,
+                                                           GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                           GTK_MESSAGE_ERROR,
+                                                           GTK_BUTTONS_CLOSE,
+                                                           _("<big><b>Failed to execute command \"%s\".</b></big>\n\n%s"),
+                                                           wavelan->command,
+                                                           error->message);
+      gtk_window_set_title (GTK_WINDOW (message_dialog), _("Error"));
+      gtk_dialog_run (GTK_DIALOG (message_dialog));
+      gtk_widget_destroy (message_dialog);
+      g_error_free (error);
+    }
+}
+
 static t_wavelan *
 wavelan_new(XfcePanelPlugin *plugin)
 {
   t_wavelan *wavelan;
-  
+  GtkSettings* settings;
+
   TRACE ("Entered wavelan_new");
-  
-	wavelan = g_new0(t_wavelan, 1);
+
+  wavelan = g_new0(t_wavelan, 1);
 
   wavelan->autohide = FALSE;
   wavelan->autohide_missing = FALSE;
 
   wavelan->signal_colors = TRUE;
   wavelan->show_icon = TRUE;
+  wavelan->show_bar = TRUE;
+  wavelan->command = g_strdup("nm-connection-editor");
   wavelan->state = -2;
 
   wavelan->plugin = plugin;
@@ -345,6 +472,7 @@ wavelan_new(XfcePanelPlugin *plugin)
   gtk_event_box_set_visible_window(GTK_EVENT_BOX(wavelan->ebox), FALSE);
   gtk_event_box_set_above_child(GTK_EVENT_BOX(wavelan->ebox), TRUE);
   g_signal_connect(wavelan->ebox, "query-tooltip", G_CALLBACK(tooltip_cb), wavelan);
+  g_signal_connect(wavelan->ebox, "button-release-event", G_CALLBACK(wavelan_icon_clicked), wavelan);
   xfce_panel_plugin_add_action_widget(plugin, wavelan->ebox);
   gtk_container_add(GTK_CONTAINER(plugin), wavelan->ebox);
 
@@ -361,7 +489,13 @@ wavelan_new(XfcePanelPlugin *plugin)
       GTK_STYLE_CONTEXT (gtk_widget_get_style_context (GTK_WIDGET (wavelan->signal))),
       GTK_STYLE_PROVIDER (wavelan->css_provider),
       GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
-  wavelan->image = GTK_WIDGET(xfce_panel_image_new_from_source("network-wireless"));
+
+  settings = gtk_settings_get_default();
+  g_signal_connect_swapped(settings, "notify::gtk-icon-theme-name", G_CALLBACK(wavelan_refresh_icons), wavelan);
+  wavelan->signal_strength = INIT;
+  wavelan_refresh_icons(wavelan);
+  wavelan->image = gtk_image_new();
+  gtk_image_set_from_icon_name (GTK_IMAGE (wavelan->image), strength_to_icon[wavelan->signal_strength], GTK_ICON_SIZE_BUTTON);
 
   gtk_box_pack_start(GTK_BOX(wavelan->box), GTK_WIDGET(wavelan->image), FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(wavelan->box), GTK_WIDGET(wavelan->signal), FALSE, FALSE, 0);
@@ -397,6 +531,9 @@ wavelan_free(XfcePanelPlugin* plugin, t_wavelan *wavelan)
   if (wavelan->interface != NULL)
     g_free(wavelan->interface);
 
+  if (wavelan->command != NULL)
+    g_free(wavelan->command);
+
   g_free(wavelan);
 }
 
@@ -428,6 +565,11 @@ wavelan_write_config(XfcePanelPlugin *plugin, t_wavelan *wavelan)
   xfce_rc_write_bool_entry (rc, "AutohideMissing", wavelan->autohide_missing);
   xfce_rc_write_bool_entry (rc, "SignalColors", wavelan->signal_colors);
   xfce_rc_write_bool_entry (rc, "ShowIcon", wavelan->show_icon);
+  xfce_rc_write_bool_entry (rc, "ShowBar", wavelan->show_bar);
+  if (wavelan->command)
+  {
+    xfce_rc_write_entry (rc, "Command", wavelan->command);
+  }
 
   xfce_rc_close(rc);
   
@@ -458,7 +600,7 @@ wavelan_set_size(XfcePanelPlugin* plugin, int size, t_wavelan *wavelan)
   border_width = size > 26 ? 2 : 1;
   wavelan->size = size;
   image_size = wavelan->size - (2 * border_width);
-  xfce_panel_image_set_size(XFCE_PANEL_IMAGE(wavelan->image), image_size);
+  gtk_image_set_pixel_size (GTK_IMAGE (wavelan->image), image_size);
   gtk_container_set_border_width(GTK_CONTAINER(wavelan->box), border_width);
   if (wavelan->orientation == GTK_ORIENTATION_HORIZONTAL)
    gtk_widget_set_size_request(wavelan->ebox, -1, wavelan->size);
@@ -470,10 +612,10 @@ wavelan_set_size(XfcePanelPlugin* plugin, int size, t_wavelan *wavelan)
 static void
 wavelan_interface_changed(GtkEntry *entry, t_wavelan *wavelan)
 {
-    if (wavelan->interface != NULL)
-          g_free(wavelan->interface);
-    wavelan->interface = g_strdup(gtk_entry_get_text(entry));
-    wavelan_reset(wavelan);
+  if (wavelan->interface != NULL)
+    g_free(wavelan->interface);
+  wavelan->interface = g_strdup(gtk_entry_get_text(entry));
+  wavelan_reset(wavelan);
 }
 
 /* autohide toggled callback */
@@ -503,6 +645,15 @@ wavelan_show_icon_changed(GtkToggleButton *button, t_wavelan *wavelan)
   wavelan_set_state(wavelan, wavelan->state);
 }
 
+/* show signal bar callback */
+static void
+wavelan_show_bar_changed(GtkToggleButton *button, t_wavelan *wavelan)
+{
+  TRACE ("Entered wavelan_show_bar_changed");
+  wavelan->show_bar = gtk_toggle_button_get_active(button);
+  wavelan_set_state(wavelan, wavelan->state);
+}
+
 /* signal colors callback */
 static void
 wavelan_signal_colors_changed(GtkToggleButton *button, t_wavelan *wavelan)
@@ -512,6 +663,15 @@ wavelan_signal_colors_changed(GtkToggleButton *button, t_wavelan *wavelan)
   wavelan_set_state(wavelan, wavelan->state);
 }
 
+/* command changed callback */
+static void
+wavelan_command_changed(GtkEntry *entry, t_wavelan *wavelan)
+{
+  if (wavelan->command != NULL)
+    g_free(wavelan->command);
+  wavelan->command = g_strdup(gtk_entry_get_text(entry));
+}
+
 static void
 wavelan_dialog_response (GtkWidget *dlg, int response, t_wavelan *wavelan)
 {
@@ -527,7 +687,7 @@ static void
 wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
 {
   GtkWidget *dlg, *hbox, *label, *interface, *vbox, *autohide;
-  GtkWidget *autohide_missing, *warn_label, *signal_colors, *show_icon;
+  GtkWidget *autohide_missing, *warn_label, *signal_colors, *show_icon, *show_bar, *command;
   GtkWidget *combo;
   GList     *interfaces, *lp;
 
@@ -547,7 +707,7 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
                     wavelan);
 
   xfce_titled_dialog_set_subtitle (XFCE_TITLED_DIALOG (dlg), _("Properties"));
-              
+
   vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 6);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
   gtk_widget_show(vbox);
@@ -609,17 +769,6 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
   gtk_box_pack_start(GTK_BOX(hbox), warn_label, TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
-  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 12);
-  gtk_widget_show(hbox);
-  signal_colors = gtk_check_button_new_with_mnemonic(_("Enable sig_nal quality colors"));
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(signal_colors), 
-      wavelan->signal_colors);
-  g_signal_connect(signal_colors, "toggled", 
-      G_CALLBACK(wavelan_signal_colors_changed), wavelan);
-  gtk_widget_show(signal_colors);
-  gtk_box_pack_start(GTK_BOX(hbox), signal_colors, TRUE, TRUE, 0);
-  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-
   hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 12);
   gtk_widget_show(hbox);
   show_icon = gtk_check_button_new_with_mnemonic(_("Show _icon"));
@@ -631,6 +780,43 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
   gtk_box_pack_start(GTK_BOX(hbox), show_icon, TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
+  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 12);
+  gtk_widget_show(hbox);
+  show_bar = gtk_check_button_new_with_mnemonic(_("Show signal _bar"));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(show_bar),
+      wavelan->show_bar);
+  g_signal_connect(show_bar, "toggled",
+      G_CALLBACK(wavelan_show_bar_changed), wavelan);
+  gtk_widget_show(show_bar);
+  gtk_box_pack_start(GTK_BOX(hbox), show_bar, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+
+  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 12);
+  gtk_widget_show(hbox);
+  signal_colors = gtk_check_button_new_with_mnemonic(_("Enable sig_nal quality colors"));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(signal_colors),
+      wavelan->signal_colors);
+  g_signal_connect(signal_colors, "toggled",
+      G_CALLBACK(wavelan_signal_colors_changed), wavelan);
+  gtk_widget_show(signal_colors);
+  gtk_box_pack_start(GTK_BOX(hbox), signal_colors, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+
+  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 12);
+  gtk_widget_show(hbox);
+  label = gtk_label_new(_("Wifi Manager Command"));
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0f);
+  gtk_widget_show(label);
+  command = gtk_entry_new();
+  if (wavelan->command != NULL)
+    gtk_entry_set_text(GTK_ENTRY(command), wavelan->command);
+  g_signal_connect(command, "changed", G_CALLBACK(wavelan_command_changed),
+      wavelan);
+  gtk_widget_show(command);
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(hbox), command, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+
   for (lp = interfaces; lp != NULL; lp = lp ->next)
     g_free (lp->data);
   g_list_free (interfaces);
